{******************* Mcga Project *****************
 * Mcga_vs - Mode 13h (320x200x256) graphics unit *
 * made by: Guy Granot and Skobin....             *
 **************************************************
}
{$A+ Align all data on word boundaries       }
{$B- Enable short circuit boolean evaluation }
{$D+ Enable debugging information            }
{$E- Disable x87 emmulation                  }
{$F- Do not force far calls                  }
{$G+ Enable 286 instructions                 }
{$I- Disable IO checking                     }
{$O- Disable overlay support                 }
{$Q- Disable overflow checking               }
{$R- Disable range checking                  }
{$S- Disable stack checking                  }
{$T- Disable type checking on pointers       }
{$V- Disable VAR type checking               }
{$X+ Enable Extended syntax                  }
{$Y- Disable symbol reference info           }
{$N+}
unit mcga_vs;
interface
  {1}
  function  getvs(var Vs:pointer):word;
  {2}
  procedure freevs(vs:pointer);
  {3}
  Procedure Setmcga; {Get in mcga 320*200*256}
  {4}
  Procedure Settext; {Get Back to text mode}
  {5}
  Procedure FlipScreen(source,destination:Word);{flips the screens}
  {6}
  Procedure Cls(color:byte;ad:word); {clears the screen to a color in mcga}
  {7}
  Procedure Putpixel(X,Y:integer; Color: Byte; ad:word); {puts a pixel on tha screen}
  {8}
  Function  Getpixel(x,y : integer;ad:word):byte; {gets a pixel from tha screen}
  {9}
  Procedure GetPal(Col : Byte; Var R,G,B : Byte); {get the pallete of a colour}
  {10}
  Procedure Setpal(col:byte; r,g,b:byte); {Set The pallete of a color}
  {11}
  Procedure WaitRetrace; {waits for a vertical retrace to prevent fuzz on the screen}
  {12}
  Procedure Blackout; {Black out the Entire Pallete}
  {13}
  Procedure Fadeout(Del:Integer); {Fade out the screen with a delay}
  {14}
  Procedure Elips(oX,oY,radx,rady:integer;Color:byte;ad:word); {paints a circle}
  {15}
  procedure Circle(oX,oY:integer;rad:real;Color:byte;ad:word); {paints a circle}
  {16}
  Procedure Line(x1,y1,x2,y2:integer;Color:byte;ad:word); {paints a line}
  {17}
  Procedure LimPixel(X,Y:integer; Color: Byte; ad:word);
  {18}
  Function LimGetpixel(x,y : integer;ad:word):byte;
  {19}
  procedure SetScreenLim(inminx,inminy,inmaxx,inmaxy:integer);

  Const VGA=$a000;

  Implementation
    Uses Crt;
    Var
     MAXX,MAXY,MINX,MINY:integer;

  function getvs(var vs:pointer):word;
  begin
    GetMem (Vs,64000);
    getvs := seg(vs^);
  end;

  procedure freevs(vs:pointer);
  begin
    FreeMem(Vs, 64000);
  end;

Procedure Setmcga; Assembler;
Asm
  Mov ax,0013h
  Int 10h
End;

Procedure Settext; Assembler;
Asm
  Mov ax,0003h
  Int 10h
End;

Procedure FlipScreen(source,destination:Word); assembler;
 { Move (source,mem [destination:0],64000);}
 Asm
   push    ds
   mov     ax, [destination]
   mov     es, ax
   mov     ax, [source]
   mov     ds, ax
   xor     si, si
   xor     di, di
   mov     cx, 32000
   rep     movsw
   pop     ds
  End;

Procedure Cls(Color:Byte;ad:word); Assembler;
{Fillchar(mem [ad:0],64000,color);}
asm
   push    es
   mov     cx, 32000;
   mov     es,[ad]
   xor     di,di
   mov     al,[Color]
   mov     ah,al
   rep     stosw
   pop     es
End;

Procedure Putpixel(X,Y: Integer; Color: Byte;ad:word);
begin
 mem[ad:y shl 8+y shl 6+x]:=color;
End;

Function Getpixel(x,y : integer;ad:word):byte;
begin
 Getpixel:=mem[ad:y shl 8+y shl 6+x];
end;

Procedure Getpal(Col:Byte; Var R,G,B: Byte);
Begin;
  Port[$3c7]:=Col;
  R:=Port[$3c9];
  G:=Port[$3c9];
  B:=Port[$3c9];
End;

Procedure Setpal(Col:Byte; R,G,B:Byte);
Begin
  Port[$3c8]:=Col;
  Port[$3c9]:=R;
  Port[$3c9]:=G;
  Port[$3c9]:=B;
End;

Procedure WaitRetrace;Assembler;
label
  l1, l2;
asm
    mov dx,3DAh
l1:
    in al,dx
    and al,08h
    jnz l1
l2:
    in al,dx
    and al,08h
    jz  l2
end;

procedure Blackout;
Var
 I:Integer;
Begin;
  Waitretrace;
  For I:=0 to 255 do
    Setpal(i,0,0,0);
End;

Procedure Fadeout(Del:Integer);
Var I,C:Integer;
    Janko2: Array [1..3] of Byte;
Begin
  For I:=1 to 64 do begin
    Delay(Del);
    Waitretrace;
    For C:=0 to 255 do begin
      Getpal(C,Janko2[1],Janko2[2],Janko2[3]);
      if Janko2[1]>0 then Dec (Janko2[1]);
      if Janko2[2]>0 then Dec (Janko2[2]);
      if Janko2[3]>0 then Dec (Janko2[3]);
      SetPal (C,Janko2[1],Janko2[2],Janko2[3]);
    End;
  End;
End;


Procedure Elips(oX,oY,radx,rady:integer; Color:byte;ad:word);
Var deg,z:real;
    x,y:integer;
begin
 deg:=0;
 if radx*rady=0 then z:=6.5
 else
 begin
  if radx>rady then z:=0.4/radx
   else z:=0.4/rady;
 end;
 repeat
   x:=round(radx*COS(deg));
   y:=round(rady*SIN(deg));
   limpixel(x+oX,y+oY,Color,ad);
   deg:=deg+z;
 until (deg>6.4);
end;

procedure Circle(oX,oY:integer;rad:real;Color:byte;ad:word); {paints a circle}
var
  i,dx,dy:integer;
begin
  if rad>0 then
  for i:=0 to round(rad*pi/2) do
  begin
    dx:=round(rad*cos(i/rad));
    dy:=round(rad*sin(i/rad));
    limpixel(Ox+dx,Oy+dy,Color,ad);
    limpixel(Ox-dx,Oy-dy,Color,ad);
    limpixel(Ox-dx,Oy+dy,Color,ad);
    limpixel(Ox+dx,Oy-dy,Color,ad);
  end;
end;

procedure line(x1,y1,x2,y2:integer; color:byte;ad:word);

    function sgn(a:real):integer;
    begin
         if a>0 then sgn:=+1;
         if a<0 then sgn:=-1;
         if a=0 then sgn:=0;
    end;

var u,s,v,d1x,d1y,d2x,d2y,m,n:real;
    i:integer;
begin
     u:= x2 - x1;
     v:= y2 - y1;
     d1x:= SGN(u);
     d1y:= SGN(v);
     d2x:= SGN(u);
     d2y:= 0;
     m:= ABS(u);
     n := ABS(v);
     IF NOT (M>N) then
     BEGIN
          d2x := 0 ;
          d2y := SGN(v);
          m := ABS(v);
          n := ABS(u);
     END;
     s := INT(m / 2);
     FOR i := 0 TO round(m) DO
     BEGIN
          limpixel(x1,y1,color,ad);
          s := s + n;
          IF not (s<m) THEN
          BEGIN
               s := s - m;
               x1:=x1+round(d1x);
               y1:=y1+ round(d1y);
          END
          ELSE
          BEGIN
               x1:=x1+round(d2x);
               y1:=y1+round(d2y);
          END;
     end;
END;

Procedure Limpixel(X,Y: Integer; Color: Byte;ad:word);
begin
  if(ad = 0)or(X > MAXX)or(Y > MAXY)or(X < MINX)or(Y < MINY)then exit;
{  if X > MAXX then exit;
  if Y > MAXY then exit;
  if X < MINX then exit;
  if Y < MINY then exit;}
  mem[ad:y shl 8+y shl 6+x]:=color;
End;

Function LimGetpixel(x,y : integer;ad:word):byte;
begin
 LimGetPixel:=0;
 if ad = 0 then exit;
 if X > MAXX then exit;
 if Y > MAXY then exit;
 if X < MINX then exit;
 if Y < MINY then exit;
 LimGetpixel:=mem[ad:y shl 8+y shl 6+x];
end;

procedure SetScreenLim(inminx,inminy,inmaxx,inmaxy:integer);
begin
   MINX:=inminx;
   MINY:=inminy;
   MAXX:=inmaxx;
   MAXY:=inmaxy;
end;

begin
   MAXX:=319;
   MAXY:=199;
   MINX:=0;
   MINY:=0;
end.